var path=require("path");
var fs=require("fs");
var glob=require("glob");
var fsExtra=require("fs.extra");
var exec=require("child_process").exec;
var winston=require("winston");

var TDPGlobFileCopier={};

TDPGlobFileCopier.run=function TDPGlobFileCopierRun(config)
{	
	// consider adding a check here to force user to be root (unless configured otherwise(?)
	// process.getuid()===0 && process.platform!=="win32"
	
	
	// Default to generic config if not included
	if(!config)
	{
		config=path.join(__dirname, "/config/TDPGlobFileCopierConfig.js");
	}
	
	var t=typeof(config);
	
	if(t==="string") // We'll assume we've been passed a path to a file
	{
		// load file into an object, this assumes it's a valid path, otherwise it'll fail here
		// The contents of this file should simply be a raw JS object
		config=require(config);
	}
	else if(t!=="object") // Error, we should receive a path to a file or a raw JS object only
	{
		return false;
	}
	
	if(this.logger=this.setupLogging(config.logging))
	{
		return this.copyFiles(config.operations);
	}
	else
	{
		return false;
	}
}

TDPGlobFileCopier.setupLogging=function TDPGlobFileCopierSetupLogging(loggingConfig)
{
	if(typeof(loggingConfig)==="object")
	{
		// defaults
		loggingConfig.transport=loggingConfig.transport || "console";
		loggingConfig.options=loggingConfig.options || {level:"warning"};
	
		var logger=new (winston.Logger)
				({
					transports: [new (winston.transports[loggingConfig.transport])(loggingConfig.options)]
				});
  
		return logger;
	}
	else
	{
		console.log("ERROR: Supplied logging config is not an object");
		return false;
	}
}

TDPGlobFileCopier.copyFiles=function TDPGlobFileCopierCopyFiles(operationsConfig)
{
	this.logger.log("debug", "Beginning TDPGlobFileCopierCopyFiles with operationsConfig: ", operationsConfig);

	if(typeof(operationsConfig)==="object")
	{
		this.logger.log("debug", "operationsConfig is an object: OK, continuing");
		
		for(var i in operationsConfig)
		{		
			this.logger.log("debug", "Processing operationsConfig "+i+": ", operationsConfig[i]);
			
	// TODO: prob should use __dirname here instead of process.cwd()		
			var source=path.join(process.cwd(), operationsConfig[i]["source"]);
			this.logger.log("debug", "Source path is "+source);
			
			var destination=path.join(process.cwd(), operationsConfig[i]["destination"]);
			this.logger.log("debug", "Destination path is "+destination);

			var files=glob.sync(source);
			this.logger.log("debug", "Glob found files: ", files);

			for(var j in files)
			{
				this.logger.log("debug", "Processing file: ", files[j]);
				
				var d=path.join(destination, files[j].replace(path.dirname(source), ""));
				this.logger.log("debug", "Destination path and filename is "+d);

				if(!fs.existsSync(destination)) // Check if destination dir exists
				{
					var c=fsExtra.mkdirRecursiveSync(destination);
					this.logger.log("debug", "Destination base dir ("+destination+") does not exist, creating: "+c);

					if(typeof(operationsConfig[i]["permissions"])==="object")
					{
						this.setPermissions(destination, operationsConfig[i]["permissions"])
						this.logger.log("debug", "Setting permissions on destination base dir ("+destination+") to: ", operationsConfig[i]["permissions"]);
					}
					else
					{
						this.logger.log("warning", "Can't set permissions on destination base dir ("+destination+"), config.operations is not an object!");
					}
				}

				// TODO: Tidy this (checking dir exists twice), it's clunky
				if(fs.existsSync(destination))
				{
					this.logger.log("debug", "Destination dir ("+destination+") exists!");
					
					var stat=fs.statSync(files[j]);

					if(stat)
					{
						if(stat.isDirectory())
						{
	// Do we really need to create dirs if we're not able to write files into them recursively?		
							var c=fsExtra.mkdirRecursiveSync(d);
							this.logger.log("debug", "Destination path ("+destination+") does not exist, creating: "+c);

							if(typeof(operationsConfig[i]["permissions"])==="object")
							{
								this.setPermissions(d, operationsConfig[i]["permissions"])
								this.logger.log("debug", "Setting permissions on destination dir ("+files[j]+") to: ", operationsConfig[i]["permissions"]);
							}
							else
							{
								this.logger.log("warning", "Can't set permissions on destination base dir ("+files[j]+"), config.operations is not an object!");
							}
						}
						else if(stat.isFile())
						{
							var data=fs.readFileSync(files[j]);

							var w=fs.writeFileSync(d, data);


					// Yes, I know, this is obscenely lazy but the permissions setting functions in node don't make it easy...will be improved soon!		
					// Also need to be much more careful with params - the below is woefully unsafe and will also be improved
							if(typeof(operationsConfig[i]["permissions"])==="object")
							{
								this.setPermissions(d, operationsConfig[i]["permissions"]);
								this.logger.log("debug", "Setting permissions on destination dir ("+files[j]+") to: ", operationsConfig[i]["permissions"]);
							}
							else
							{
								this.logger.log("warning", "Can't set permissions on destination base dir ("+files[j]+"), config.operations is not an object!");
							}
						}
					}

				}
				else
				{
					this.logger.log("error", "Could not create "+destination+", could not copy "+files[j]+" because the base dir could not be created");
				}
			}
		}
	}
	else
	{
		this.logger.log("debug", "operationsConfig is not an object: FAIL");
		return false;
	}
}


TDPGlobFileCopier.setPermissions=function TDPGlobFileCopiersetPermissions(file, permissions)
{
	this.logger.log("debug", "Setting permissions on "+file+" to: ", permissions);

// TODO: Fix this check:
	if(1==0)//fs.existsSync(file))
	{
		this.logger.log("error", "File ("+file+") does not exist!");
		return false;
	}
	else if(typeof(permissions)!=="object")
	{
		this.logger.log("error", "permissions parameter is not an object");
		return false;
	}
	else if(process.getuid()===0 && process.platform!=="win32")
	{
		if(permissions.user)
		{			
			exec("chown -R "+permissions.user.toString()+" "+file);
			this.logger.log("debug", "Setting user for "+file+" to: ", permissions.user);
		}
		if(permissions.group)
		{								
			exec("chgrp -R "+permissions.group.toString()+" "+file);
			this.logger.log("debug", "Setting group for "+file+" to: ", permissions.group);
		}
		if(permissions.mode)
		{								
			exec("chmod -R "+permissions.mode.toString()+" "+file);
			this.logger.log("debug", "Setting mode for "+file+" to: ", permissions.mode);
		}
		
	// This is a bit ridiculous as the exec's are async...	
		return true; 
	}
	else
	{
		this.logger.log("error", "Can't set permissions for "+file);
		return false;
	}
}

module.exports=TDPGlobFileCopier;